Screen {
    overflow: auto;
    align: center middle;
}

Button {
    color: white;
    background: $background-lighten-2;
    border-bottom: tall $accent;
    min-width: 15;
    margin-left: 1;
}

#prev_button,
#rewind_button,
#ffwd_button,
#next_button {
    min-width: 7;
}

#play_pause_button {
    border-bottom: tall $success;
}

#stop_button {
    border-bottom: tall $error;
}

Bar > .bar--bar {
    color: $accent;
    background: $background-lighten-3;
}

#volume_buttons,
#playback_buttons,
#wind_buttons {
    layout: horizontal;
    align: center middle;
    height: 3;
    width: 32;
    min-width: 14;
    margin-top: 1;
}

#controls {
    layout: vertical;
    align: center top;
    width: 100%;
    height: 15;
    margin-left: 1;
    margin-right: 1;
    border: grey;
    /*background: red;*/
}

#progress {
    layout: vertical;
    align: center middle;
    width: 100%;
    margin-left: 1;
    margin-right: 1;
    padding: 0;
    height: 4;
    border: grey;
    text-style: bold;
}

#progress_label {
    margin-left: 7;
}

#device_info,
#playback_info {
    height: 3;
    width: 100%;
    align: center middle;
    margin-top: 0;
    margin-bottom: 0;
    margin-right: 1;
    margin-left: 1;
    border: grey;
    text-style: bold;
}

#url_input_container,
LolCatUrlInput {
    layout: horizontal;
    height: 5;
    width: 100%;
    align: center middle;
    margin-top: 0;
    margin-bottom: 0;
    margin-right: 1;
    margin-left: 1;
    border: grey;
}

#app {
    layout: vertical;
    align: center middle;
    dock: top;
    max-width: 60;
}
